import logging
import os
from typing import Any

import configargparse

config = None


def get_config() -> Any:
    parser = configargparse.get_argument_parser(
        config_file_parser_class=configargparse.YAMLConfigFileParser,
        formatter_class=configargparse.ArgumentDefaultsHelpFormatter,
    )

    # parser.add_argument("--step", action="store_true", default=False, help="step mode")

    parser.add_argument(
        "--debug", action="store_true", default=False, help="debugging mode"
    )
    parser.add_argument(
        "-c", "--config", required=False, is_config_file=True, help="config file path"
    )
    parser.add_argument(
        "--local", required=False, is_config_file=True, help="local config file path"
    )

    parser.add_argument(
        "--config-save",
        required=False,
        is_write_out_config_file_arg=True,
        help="config file path",
    )

    # parser.add_argument("images", nargs="*", default=[], help="list of images")
    parser.add_argument("--show", action="store_true", default=False)
    parser.add_argument("--dummy", action="store_true", default=False)
    parser.add_argument("--roi", action="store_true", default=False, help="show roi")
    parser.add_argument(
        "--rtsp", action="store_true", default=False, help="activate rtsp streaming"
    )
    parser.add_argument(
        "--rtspold",
        action="store_true",
        default=False,
        help="activate rtsp streaming (old method)",
    )
    parser.add_argument("--rtsp-port", type=int, default=8554, help="rtsp port")
    parser.add_argument(
        "--rtsp-mount", default="/ds-test", help="rtsp mount point, default /ds"
    )
    parser.add_argument(
        "--codec",
        default="h265",
        help="file codec h264/h265, default h265",
        choices=["h264", "h265"],
    )
    parser.add_argument(
        "--privacy", action="store_true", default=False, help="blur plates"
    )
    parser.add_argument(
        "--key", action="store_true", default=False, help="wait for a keyboard key"
    )
    parser.add_argument(
        "--pause", action="store_true", default=False, help="start with paused pipeline"
    )
    parser.add_argument("--osd", action="store_true", default=False, help="enable osd")
    parser.add_argument(
        "--experimental",
        action="store_true",
        default=False,
        help="test experimental code",
    )
    parser.add_argument(
        "--hide-persons",
        action="store_true",
        default=False,
        help="blur person",
    )
    parser.add_argument(
        "--new-nvstreammux",
        action="store_true",
        default=True,
        help="use new nvstreammux 2",
    )
    parser.add_argument(
        "-p", "--pipeline", default="default", help="pipeline to execute"
    )
    parser.add_argument(
        "--width", default=2200, type=int, help="image width resolution"
    )
    parser.add_argument(
        "--height", default=1800, type=int, help="image height resolution"
    )
    parser.add_argument(
        "--bitrate", default=4000000, type=int, help="h26x encoder bitrate"
    )
    parser.add_argument(
        "--exposure", default=500, type=float, help="exposure time in microseconds"
    )
    parser.add_argument("--gain-max", default=6.0, type=float, help="max camera gain")
    parser.add_argument(
        "--target-brightness", default=0.2, type=float, help="max camera gain"
    )
    parser.add_argument(
        "--min-distance",
        default=0.005,
        type=float,
        help="min distance between detections",
    )
    parser.add_argument("--framerate", default=30, type=int, help="framerate")
    parser.add_argument("--zoom", default=1.0, type=float, help="display zoom")
    parser.add_argument("--step", default=0, type=int, help="step in ms, default 0")
    parser.add_argument("--tracker-width", default=640, type=int)
    parser.add_argument("--tracker-height", default=0, type=int)  # auto
    parser.add_argument(
        "--tracker-config",
        default="trackers/config_tracker_NvSORT.yml",
        help="yaml for specific tracker",
    )
    parser.add_argument("--num-sources", default=0, type=int, help="")
    parser.add_argument("--save", default="", help="save mpeg video to path")
    parser.add_argument("--save-frames", default="", help="save image path")
    parser.add_argument(
        "--save-context", default="/tmp/images", help="save image context path"
    )
    parser.add_argument(
        "--save-async",
        action="store_true",
        default=False,
        help="save context images in thread",
    )
    parser.add_argument("--save-wamp", default="", help="save wamp messages to file")
    parser.add_argument("--max-frames", type=int, default=250, help="save max frames")
    parser.add_argument(
        "--jpeg-quality", type=int, default=90, help="jpeg quality 0-100, default=90"
    )

    parser.add_argument(
        "--file1",
        default="/opt/nvidia/deepstream/deepstream/samples/streams/sample_720p.h264",
        help="sample file",
    )
    parser.add_argument(
        "--video0", default="/samples/famila/0.mkv", help="sample file camera 0"
    )
    parser.add_argument(
        "--video1", default="/samples/famila/1.mkv", help="sample file camera 1"
    )
    parser.add_argument("--video", default="", help="sample file camera 0 and 1")
    # parser.add_argument("--camera0", default="40433901", help="serial of camera 0")
    # parser.add_argument("--camera1", default="40421511", help="serial of camera 1")
    parser.add_argument("--camera", action="append")
    parser.add_argument("--save", default="", help="save folder")
    parser.add_argument(
        "--wamp", action="store_true", default=True, help="activate wamp"
    )
    parser.add_argument("--ws", default="ws://nexus:8082")
    parser.add_argument("--realm", default="realm1")

    parser.add_argument("--socket-ip-gps", default="brain")
    parser.add_argument("--socket-port-gps", type=int, default=5012)
    parser.add_argument(
        "--gps-sync",
        action="store_true",
        default=False,
        help="emit GPS sync on UDP port --socket-port-gps",
    )
    parser.add_argument(
        "--plate-angle-vertical",
        type=float,
        default=10,
        help="limit between parking vertical and parallel. It depends from camera height",
    )
    parser.add_argument(
        "--plates-truth", default="", help="path to load file with plates"
    )
    parser.add_argument("--gpx", default="", help="path to gpx track file")
    parser.add_argument(
        "--score-min",
        default=0.7,
        type=float,
        help="filter plates with greater score thant score-min (default 0.7)",
    )
    parser.add_argument("--scene", default="/samples/famila")

    global config
    config, _ = parser.parse_known_args()
    logging.basicConfig(
        format="%(asctime)s [%(levelname)s] %(name)s %(message)s",
        level="DEBUG" if config.debug else "INFO",
    )

    if config.step < 0:
        config.key = True

    if config.key:
        config.step = 1
        config.show = True
        config.osd = True

    log = logging.getLogger(__name__)
    if not config.tracker_height:
        config.tracker_height = int(
            config.tracker_width / (config.width / config.height)
        )
        log.debug(
            "set tracker shape to %s", (config.tracker_width, config.tracker_height)
        )

    if config.save_context and not os.path.exists(config.save_context):
        os.makedirs(config.save_context)
        log.info("creating %s", config.save_context)

    if os.getenv("USE_NEW_NVSTREAMMUX") == "yes":
        config.new_nvstreammux = True

    if config.new_nvstreammux:
        log.info("enabling new nvstreammux")

    # if config.rtsp and config.save_context:
    #     log.error("rtsp and save-context are incomplatibles")
    #     sys.exit(1)

    if config.video:
        config.video0 = f"{config.video}_0.mkv"
        config.video1 = f"{config.video}_1.mkv"
        config.plates_truth = f"{config.video}_truth.txt"

    if config.scene:
        config.video0 = os.path.join(config.scene, "0.mkv")
        video1 = os.path.join(config.scene, "1.mkv")
        if os.path.exists(video1):
            config.video1 = video1
        config.plates_truth = os.path.join(config.scene, "truth.txt")
        config.gpx = os.path.join(config.scene, "track.gpx")

    if config.debug:
        import jurigged

        logging.getLogger("watchdog").setLevel(logging.INFO)
        jurigged.watch(pattern="*mev_yolodeep*.py")

    return config
