################################################################################
# Copyright (c) 2019-2020, NVIDIA CORPORATION. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
################################################################################

import logging
import time

log = logging.getLogger(__name__)
start_time = time.time()
frame_count = 0


class FPSMonitor:
    def __init__(self, stream_id):
        global start_time
        self.start_time = start_time
        self.is_first = True
        global frame_count
        self.frame_count = frame_count
        self.stream_id = stream_id
        self.fps_mean = 0
        self.n = 0

    def get_fps(self, poll=1.0, show=False):
        end_time = time.time()
        if self.is_first:
            self.start_time = end_time
            self.is_first = False
        if show or end_time - self.start_time > poll:
            fps = float(self.frame_count) / poll
            log.debug(
                "FPS of stream %s is %s mean=%s", self.stream_id, fps, self.fps_mean
            )
            self.frame_count = 0
            self.start_time = end_time
            self.n += 1
            self.fps_mean = self.fps_mean * (self.n - 1) / (self.n) + fps / self.n
        else:
            self.frame_count = self.frame_count + 1

    def print_data(self):
        print("frame_count=", self.frame_count)
        print("start_time=", self.start_time)
