import asset

from mev_yolodeep.lib.pipeline import Pipeline


class DemoPipeline(Pipeline):
    def setup(self):
        if self.config.video0:
            self.create_source_bin(index=0, video_uri="file://" + self.config.video0)
        if self.config.video1:
            self.create_source_bin(index=1, video_uri="file://" + self.config.video1)

        self.streammux = self.create_streammux(2000, 1748)
        if not self.config.save_frames:
            self.create_element(
                "queue"
            )  # risolve i blocchi saltuari della pipeline allo start

        pgie = self.create_element("nvinfer", "primary-inference", "PGIE")
        pgie.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-plates_config.txt").filename,
        )
        pgie_batch_size = pgie.get_property("batch-size")
        if pgie_batch_size != self.num_sources:
            pgie.set_property("batch-size", self.num_sources)

        self.create_tracker(
            asset.load("mev_yolodeep:assets/trackers/tracker.txt").filename
        )
        self.create_convert(format="RGBA")

        sgie1 = self.create_element("nvinfer", "sgie1")
        sgie1.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-ocr_config.txt").filename,
        )
        sgie1.set_property("batch-size", 8)

        sgie2 = self.create_element("nvinfer", "sgie2")
        sgie2.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-cls_config.txt").filename,
        )
        sgie2.set_property("batch-size", 8)

        if self.config.key or self.config.rtsp or self.config.save:
            tiler = self.create_tiler()
            self.add_inference_probe(tiler.get_static_pad("sink"))

            self.create_element("nvvideoconvert")
            if self.config.osd:
                self.create_element("nvdsosd")

            if self.config.show:
                self.create_screen_out()
            else:
                if self.config.rtsp:
                    self.create_rtsp_out()
                elif self.config.save:
                    self.create_file_out()
                else:
                    self.create_fake()

        else:
            demux = self.create_streamdemux()
            self.add_inference_probe(demux.get_static_pad("sink"))

            # def _out(source_id, pad):
            #     queue = self._element(
            #         "queue"
            #     )  # risolve i blocchi saltuari della pipeline durante lo start
            #     if self.config.osd:
            #         osd = self._element("nvdsosd")
            #         queue.link(osd)

            #     if self.config.show:
            #         out = self.create_screen_out()
            #     elif self.config.save_frames:
            #         out = self.create_jpeg_out(source_id=source_id)
            #     else:
            #         out = self.create_fake(link=False)

            #     pad.link(queue.get_static_pad("sink"))
            #     if self.config.osd:
            #         osd.link(out)
            #     else:
            #         queue.link(out)
            def _out(source_id, pad):
                if self.config.osd:
                    osd = self._element("nvdsosd")
                    pad.link(osd.get_static_pad("sink"))

                if self.config.show:
                    out = self.create_screen_out()
                elif self.config.save_frames:
                    out = self.create_jpeg_out(source_id=source_id)
                else:
                    out = self.create_fake(link=False)

                if self.config.osd:
                    osd.link(out)
                else:
                    pad.link(out.get_static_pad("sink"))

            if self.config.video0:
                _out(source_id=0, pad=demux.get_request_pad("src_0"))
            if self.config.video1:
                _out(source_id=1, pad=demux.get_request_pad("src_1"))


def run(config):
    pipeline = DemoPipeline(config=config)
    pipeline.run()
