"""

gst-launch-1.0 pylonsrc device-index=0 cam::AutoFunctionProfile=MinimizeExposureTime \
    ! "video/x-raw,width=2448,height=2048,framerate=30/1,format=BGR" \
    ! videoconvert \
    ! nvvidconv \
    ! m.sink_0 nvstreammux name=m batch-size=1\
    ! nvrtspoutsinkbin bitrate=10000000

# attuale configurazione in pipeline.py
gst-launch-1.0 pylonsrc \
    device-index=0 \
    cam::ExposureAuto=Continuous \
    cam::AutoFunctionProfile=MinimizeExposureTime \
    cam::GainAuto=Continuous \
    cam::AutoGainUpperLimit=6.0 \
    cam::AutoTargetBrightness=0.2 \
    cam::AutoFunctionROIUseBrightness-ROI1=1 \
    ! "video/x-raw(memory:NVMM),width=2448,height=2048,framerate=30/1,format=YUY2" \
    ! nvvideoconvert \
    ! nvjpegenc quality=85 \
    ! multifilesink location=/tmp/frames/%07d.jpg max-files=10


gst-launch-1.0 pylonsrc device-index=0 cam::Gain=10.3 cam::ExposureTime=60000 ! "video/x-raw,width=1024,height=1024,framerate=5/1,format=GRAY8"  !  videoconvert ! nvvidconv  !  nvrtspoutsinkbin codec=h265

gst-launch-1.0 pylonsrc device-index=0 cam::AutoFunctionProfile=MinimizeGain ! "video/x-raw,width=1024,height=1024,framerate=5/1,format=GRAY8"  !  videoconvert ! nvvidconv  !  nvrtspoutsinkbin codec=h265

nvvidconv non è un plugin deepstream

su amd64
gst-launch-1.0 pylonsrc device-index=0 cam::AutoFunctionProfile=MinimizeExposureTime \                                                                24s 
    ! "video/x-raw,width=2448,height=2048,framerate=30/1,format=YUY2" \
    ! videoconvert ! autovideosink
gst-launch-1.0 pylonsrc device-index=0 cam::AutoFunctionProfile=MinimizeExposureTime \                                                                    
    ! "video/x-raw,width=2448,height=2048,framerate=30/1,format=YUY2" \
    ! videoconvert ! nvjpegenc quality=100 ! multifilesink location=/tmp/%07d.jpg max-files=10


"""

import asset
from mev_yolodeep.lib.pipeline import Pipeline


class MevPipeline(Pipeline):
    def setup(self):
        self.create_basler(index=0, serial=self.config.camera[0])
        self.create_basler(index=1, serial=self.config.camera[1])
        self.streammux = self.create_streammux(live=True)

        pgie = self.create_element("nvinfer", "primary-inference", "PGIE")
        pgie.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-plates_config.txt").filename,
        )
        pgie_batch_size = pgie.get_property("batch-size")
        if pgie_batch_size != self.num_sources:
            pgie.set_property("batch-size", self.num_sources)

        self.create_tracker(
            asset.load("mev_yolodeep:assets/trackers/tracker.txt").filename
        )
        self.create_convert(format="RGBA")

        sgie1 = self.create_element("nvinfer", "sgie1")
        sgie1.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-ocr_config.txt").filename,
        )
        sgie1.set_property("batch-size", 8)

        sgie2 = self.create_element("nvinfer", "sgie2")
        sgie2.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-cls_config.txt").filename,
        )
        sgie2.set_property("batch-size", 8)

        if self.config.key or self.config.rtsp or self.config.save:
            tiler = self.create_tiler()
            self.add_inference_probe(tiler.get_static_pad("sink"))

            self.create_element("nvvideoconvert")
            if self.config.osd:
                self.create_element("nvdsosd")

            if self.config.show:
                self.create_screen_out()
            else:
                if self.config.rtsp:
                    self.create_rtsp_out()
                elif self.config.save:
                    self.create_file_out()
                else:
                    self.create_fake()

        else:
            demux = self.create_streamdemux()
            self.add_inference_probe(demux.get_static_pad("sink"))

            def _out(source_id, pad):
                queue = self._element(
                    "queue"
                )  # risolve i blocchi saltuari della pipeline durante lo start
                if self.config.osd:
                    osd = self._element("nvdsosd")
                    queue.link(osd)

                if self.config.save_frames:
                    out = self.create_jpeg_out(source_id=source_id)
                else:
                    out = self.create_fake(link=False)

                pad.link(queue.get_static_pad("sink"))
                if self.config.osd:
                    osd.link(out)
                else:
                    queue.link(out)

            _out(source_id=0, pad=demux.get_request_pad("src_0"))
            _out(source_id=1, pad=demux.get_request_pad("src_1"))


def run(config):
    pipeline = MevPipeline(config=config)
    pipeline.run()
