
import asset
from mev_yolodeep.lib.pipeline import Pipeline


class SimplePipeline(Pipeline):
    def setup(self):
        if self.config.video0:
            self.create_source_bin(index=0, video_uri="file://" + self.config.video0)
        if self.config.video1:
            self.create_source_bin(index=1, video_uri="file://" + self.config.video1)
        self.streammux = self.create_streammux(2000, 1748)
        # self.streammux.set_property("batch-size", 100)

        pgie = self.create_element("nvinfer", "primary-inference", "PGIE")
        pgie.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-plates_config.txt").filename,
        )
        pgie_batch_size = pgie.get_property("batch-size")
        if pgie_batch_size != self.num_sources:
            pgie.set_property("batch-size", self.num_sources)

        self.create_tracker(
            asset.load("mev_yolodeep:assets/trackers/tracker.txt").filename
        )
        # Use convertor to convert from NV12 to RGBA (easier to work with in Python)
        self.create_element("nvvideoconvert")
        self.create_caps("video/x-raw(memory:NVMM), format=RGBA")

        sgie1 = self.create_element("nvinfer", "sgie1")
        sgie1.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-ocr_config.txt").filename,
        )
        sgie1.set_property("batch-size", 8)

        sgie2 = self.create_element("nvinfer", "sgie2")
        sgie2.set_property(
            "config-file-path",
            asset.load("mev_yolodeep:assets/eu-cls_config.txt").filename,
        )
        sgie2.set_property("batch-size", 8)

        tiler = self.create_tiler()
        self.create_element("nvvideoconvert")

        self.create_element("nvdsosd")
        # self.create_element("nvvideoconvert")

        if self.config.show:
            self.create_screen_out()
        else:
            if self.config.rtsp:
                self.create_rtsp_out()
            elif self.config.rtspold:
                self.create_rtsp_out_old()
            elif self.config.save:
                self.create_file_out()
            else:
                self.create_fake()

        self.add_inference_probe(tiler.get_static_pad("sink"))

        #def on_detections(self, detections: List[Detection], frame=None):
        #self.collector.add(detections, frame=frame)


def run(config):
    pipeline = SimplePipeline(config=config)
    pipeline.run()
