import sys

# sys.path.append("../")
import gi

gi.require_version("Gst", "1.0")
import argparse

from gi.repository import GLib, Gst

from mev_yolodeep.common.bus_call import bus_call
from mev_yolodeep.common.is_aarch_64 import is_aarch64


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--show",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--dummy",
        action="store_true",
        default=False,
    )
    return parser.parse_args()


def main(config):
    Gst.init(None)
    pipeline = Gst.Pipeline()

    if not pipeline:
        sys.stderr.write(" Unable to create Pipeline \n")

    if config.dummy:
        source = Gst.ElementFactory.make("videotestsrc", "testsource")
        if not source:
            sys.stderr.write(" Unable to create Source \n")
    else:
        source = Gst.ElementFactory.make("pylonsrc", "testsource")
        if not source:
            sys.stderr.write(" Unable to create Source \n")
        source.set_property("device-index", 0)

    if not config.show:
        sink = Gst.ElementFactory.make("nvrtspoutsinkbin", "rtsp")
        sink.set_property("bitrate", 4000000)
        sink.set_property("codec", "h265")
        sink.set_property("rtsp-port", 8554)
        sink.set_property("profile", "main")
    else:
        if is_aarch64():
            print("Creating nv3dsink \n")
            sink = Gst.ElementFactory.make("nv3dsink", "nv3d-sink")
            sink.set_property("sync", 0)
            if not sink:
                sys.stderr.write(" Unable to create nv3dsink \n")
        else:
            print("Creating EGLSink \n")
            sink = Gst.ElementFactory.make("nveglglessink", "nvvideo-renderer")
            sink.set_property("sync", 0)
            if not sink:
                sys.stderr.write(" Unable to create egl sink \n")

    pipeline.add(source)
    pipeline.add(sink)

    source.link(sink)

    # sinkpad = streammux.get_request_pad("sink_0")
    # if not sinkpad:
    #    sys.stderr.write(" Unable to get the sink pad of streammux \n")
    # srcpad = decoder.get_static_pad("src")
    # if not srcpad:
    #    sys.stderr.write(" Unable to get source pad of decoder \n")
    # srcpad.link(sinkpad)

    # create an event loop and feed gstreamer bus mesages to it
    loop = GLib.MainLoop()
    bus = pipeline.get_bus()
    bus.add_signal_watch()
    bus.connect("message", bus_call, loop)

    pipeline.set_state(Gst.State.PLAYING)
    sys.stdout.write("play\n")

    try:
        loop.run()
    except Exception as e:
        1 / 0
        sys.stdout.write("error %e\n" % e)
    sys.stdout.write("xxxx\n" * 10)
    pipeline.set_state(Gst.State.NULL)


def run():
    config = parse_args()
    sys.exit(main(config))


if __name__ == "__main__":
    run()
