import sys

sys.path.append("../")
import os

import gi

gi.require_version("Gst", "1.0")
import argparse
import configparser

import asset
import pyds
from gi.repository import GLib, Gst

from mev_yolodeep.common.bus_call import bus_call
from mev_yolodeep.common.FPS import PERF_DATA
from mev_yolodeep.common.is_aarch_64 import is_aarch64

PGIE_CLASS_ID_VEHICLE = 0
PGIE_CLASS_ID_BICYCLE = 1
PGIE_CLASS_ID_PERSON = 2
PGIE_CLASS_ID_ROADSIGN = 3

perf_data = None


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--show",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--video",
        default="/opt/nvidia/deepstream/deepstream/samples/streams/sample_720p.h264",
    )
    return parser.parse_args()


def osd_sink_pad_buffer_probe(pad, info, u_data):
    global perf_data
    perf_data.update_fps("stream0")

    # return  Gst.PadProbeReturn.OK

    frame_number = 0
    num_rects = 0

    gst_buffer = info.get_buffer()
    if not gst_buffer:
        print("Unable to get GstBuffer ")
        return

    # Retrieve batch metadata from the gst_buffer
    # Note that pyds.gst_buffer_get_nvds_batch_meta() expects the
    # C address of gst_buffer as input, which is obtained with hash(gst_buffer)
    batch_meta = pyds.gst_buffer_get_nvds_batch_meta(hash(gst_buffer))
    l_frame = batch_meta.frame_meta_list
    while l_frame is not None:
        try:
            # Note that l_frame.data needs a cast to pyds.NvDsFrameMeta
            # The casting is done by pyds.NvDsFrameMeta.cast()
            # The casting also keeps ownership of the underlying memory
            # in the C code, so the Python garbage collector will leave
            # it alone.
            frame_meta = pyds.NvDsFrameMeta.cast(l_frame.data)
        except StopIteration:
            break

        # Intiallizing object counter with 0.
        obj_counter = {
            PGIE_CLASS_ID_VEHICLE: 0,
            PGIE_CLASS_ID_PERSON: 0,
            PGIE_CLASS_ID_BICYCLE: 0,
            PGIE_CLASS_ID_ROADSIGN: 0,
        }

        frame_number = frame_meta.frame_num
        num_rects = frame_meta.num_obj_meta
        l_obj = frame_meta.obj_meta_list
        while l_obj is not None:
            try:
                # Casting l_obj.data to pyds.NvDsObjectMeta
                obj_meta = pyds.NvDsObjectMeta.cast(l_obj.data)
            except StopIteration:
                break

            if obj_meta.class_id not in obj_counter:
                obj_counter[obj_meta.class_id] = 0
            obj_counter[obj_meta.class_id] += 1
            obj_meta.rect_params.border_color.set(
                0.0, 0.0, 1.0, 0.8
            )  # 0.8 is alpha (opacity)
            try:
                l_obj = l_obj.next
            except StopIteration:
                break

            id = obj_meta.object_id

            l_classifier = obj_meta.classifier_meta_list
            while l_classifier is not None:
                try:
                    class_meta = pyds.NvDsClassifierMeta.cast(l_classifier.data)
                except StopIteration:
                    break
                try:
                    l_classifier = l_classifier.next
                except StopIteration:
                    break

                l_label = class_meta.label_info_list
                while l_label is not None:
                    try:
                        label_info = pyds.NvDsLabelInfo.cast(l_label.data)
                    except:
                        break
                    try:
                        l_label = l_label.next
                    except:
                        pass

                    print(f"\n Car{id} is {label_info.result_label}\n")

        stream_index = f"stream{frame_meta.pad_index}"
        perf_data.update_fps(stream_index)

        # Acquiring a display meta object. The memory ownership remains in
        # the C code so downstream plugins can still access it. Otherwise
        # the garbage collector will claim it when this probe function exits.
        display_meta = pyds.nvds_acquire_display_meta_from_pool(batch_meta)
        display_meta.num_labels = 1
        py_nvosd_text_params = display_meta.text_params[0]
        # Setting display text to be shown on screen
        # Note that the pyds module allocates a buffer for the string, and the
        # memory will not be claimed by the garbage collector.
        # Reading the display_text field here will return the C address of the
        # allocated string. Use pyds.get_string() to get the string content.
        py_nvosd_text_params.display_text = f"Frame Number={frame_number} Number of Objects={num_rects} Vehicle_count={obj_counter[PGIE_CLASS_ID_VEHICLE]} Person_count={obj_counter[PGIE_CLASS_ID_PERSON]}"

        # Now set the offsets where the string should appear
        py_nvosd_text_params.x_offset = 10
        py_nvosd_text_params.y_offset = 12

        # Font , font-color and font-size
        py_nvosd_text_params.font_params.font_name = "Serif"
        py_nvosd_text_params.font_params.font_size = 10
        # set(red, green, blue, alpha); set to White
        py_nvosd_text_params.font_params.font_color.set(1.0, 1.0, 1.0, 1.0)

        # Text background color
        py_nvosd_text_params.set_bg_clr = 1
        # set(red, green, blue, alpha); set to Black
        py_nvosd_text_params.text_bg_clr.set(0.0, 0.0, 0.0, 1.0)
        # Using pyds.get_string() to get display_text as string
        print(pyds.get_string(py_nvosd_text_params.display_text))
        pyds.nvds_add_display_meta_to_frame(frame_meta, display_meta)
        try:
            l_frame = l_frame.next
        except StopIteration:
            break

        # ------ Start of tracker test code ------ #
        # past tracking meta data
        l_user = batch_meta.batch_user_meta_list
        while l_user is not None:
            try:
                # Note that l_user.data needs a cast to pyds.NvDsUserMeta
                # The casting is done by pyds.NvDsUserMeta.cast()
                # The casting also keeps ownership of the underlying memory
                # in the C code, so the Python garbage collector will leave
                # it alone
                user_meta = pyds.NvDsUserMeta.cast(l_user.data)
            except StopIteration:
                break
            if (
                user_meta
                and user_meta.base_meta.meta_type
                == pyds.NvDsMetaType.NVDS_TRACKER_PAST_FRAME_META
            ):
                try:
                    # Note that user_meta.user_meta_data needs a cast to pyds.NvDsPastFrameObjBatch
                    # The casting is done by pyds.NvDsPastFrameObjBatch.cast()
                    # The casting also keeps ownership of the underlying memory
                    # in the C code, so the Python garbage collector will leave
                    # it alone
                    pPastFrameObjBatch = pyds.NvDsPastFrameObjBatch.cast(
                        user_meta.user_meta_data
                    )
                except StopIteration:
                    break
                # for trackobj in pyds.NvDsPastFrameObjBatch.list(pPastFrameObjBatch):
                #     print("streamId=",trackobj.streamID)
                #     print("surfaceStreamID=",trackobj.surfaceStreamID)
                #     for pastframeobj in pyds.NvDsPastFrameObjStream.list(trackobj):
                #         print("numobj=",pastframeobj.numObj)
                #         print("uniqueId=",pastframeobj.uniqueId)
                #         print("classId=",pastframeobj.classId)
                #         print("objLabel=",pastframeobj.objLabel)
                #         for objlist in pyds.NvDsPastFrameObjList.list(pastframeobj):
                #             print('frameNum:', objlist.frameNum)
                #             print('tBbox.left:', objlist.tBbox.left)
                #             print('tBbox.width:', objlist.tBbox.width)
                #             print('tBbox.top:', objlist.tBbox.top)
                #             print('tBbox.right:', objlist.tBbox.height)
                #             print('confidence:', objlist.confidence)
                #             print('age:', objlist.age)
            try:
                l_user = l_user.next
            except StopIteration:
                break
        # ------ End of tracker test code ------ #
    return Gst.PadProbeReturn.OK


def main(config):
    global perf_data
    perf_data = PERF_DATA(1)  # number of sources

    Gst.init(None)
    print("Creating Pipeline \n ")
    pipeline = Gst.Pipeline()

    if not pipeline:
        sys.stderr.write(" Unable to create Pipeline \n")

    # Source element for reading from the file
    source = Gst.ElementFactory.make("filesrc", "file-source")
    if not source:
        sys.stderr.write(" Unable to create Source \n")

    # Since the data format in the input file is elementary h264 stream, we need a h264parser
    h264parser = Gst.ElementFactory.make("h264parse", "h264-parser")
    if not h264parser:
        sys.stderr.write(" Unable to create h264 parser \n")

    # Use nvdec_h264 for hardware accelerated decode on GPU
    decoder = Gst.ElementFactory.make("nvv4l2decoder", "nvv4l2-decoder")
    if not decoder:
        sys.stderr.write(" Unable to create Nvv4l2 Decoder \n")

    # Create nvstreammux instance to form batches from one or more sources.
    streammux = Gst.ElementFactory.make("nvstreammux", "Stream-muxer")
    if not streammux:
        sys.stderr.write(" Unable to create NvStreamMux \n")

    # Use nvinfer to run inferencing on decoder's output,
    pgie = Gst.ElementFactory.make("nvinfer", "primary-inference")
    if not pgie:
        sys.stderr.write(" Unable to create pgie \n")

    # ------ Start of tracker test code ------ #
    tracker = Gst.ElementFactory.make("nvtracker", "tracker")
    if not tracker:
        sys.stderr.write(" Unable to create tracker \n")
    # ------ End of tracker test code ------ #

    # ------ Secondary Classifier 1 ------#
    sgie1 = Gst.ElementFactory.make("nvinfer", "secondary1-nvinference-engine")
    if not sgie1:
        sys.stderr.write(" Unable to make sgie1 \n")
    # ------ Secondary Classifier 1 ------#

    # ------ Secondary Classifier 2 ------#
    sgie2 = Gst.ElementFactory.make("nvinfer", "secondary2-nvinference-engine")
    if not sgie2:
        sys.stderr.write(" Unable to make sgie2 \n")
    # ------ Secondary Classifier 2 ------#

    # ------ Secondary Classifier 3 ------#
    sgie3 = Gst.ElementFactory.make("nvinfer", "secondary3-nvinference-engine")
    if not sgie3:
        sys.stderr.write(" Unable to make sgie3 \n")
    # ------ Secondary Classifier 3 ------#

    # Use convertor to convert from NV12 to RGBA as required by nvosd
    nvvidconv = Gst.ElementFactory.make("nvvideoconvert", "convertor")
    if not nvvidconv:
        sys.stderr.write(" Unable to create nvvidconv \n")

    # Create OSD to draw on the converted RGBA buffer
    nvosd = Gst.ElementFactory.make("nvdsosd", "onscreendisplay")

    if not nvosd:
        sys.stderr.write(" Unable to create nvosd \n")

    # Finally render the osd output
    if not config.show:
        print("Creating Fakesink \n")
        sink = Gst.ElementFactory.make("fakesink", "fakesink")
        sink.set_property("enable-last-sample", 0)
        sink.set_property("sync", 0)
    else:
        if is_aarch64():
            print("Creating nv3dsink \n")
            sink = Gst.ElementFactory.make("nv3dsink", "nv3d-sink")
            sink.set_property("sync", 0)
            if not sink:
                sys.stderr.write(" Unable to create nv3dsink \n")
        else:
            print("Creating EGLSink \n")
            sink = Gst.ElementFactory.make("nveglglessink", "nvvideo-renderer")
            sink.set_property("sync", 0)
            if not sink:
                sys.stderr.write(" Unable to create egl sink \n")

    print("Playing file %s " % config.video)
    source.set_property("location", config.video)
    if (
        os.environ.get("USE_NEW_NVSTREAMMUX") != "yes"
    ):  # Only set these properties if not using new gst-nvstreammux
        streammux.set_property("width", 1280)
        streammux.set_property("height", 720)
        streammux.set_property("batched-push-timeout", 4000000)

    streammux.set_property("batch-size", 1)
    pgie.set_property(
        "config-file-path",
        asset.load("mev_yolodeep:assets/helloworld_pgie_config.txt").filename,
    )

    # ------ Secondary Classifier 1 ------#
    sgie1.set_property(
        "config-file-path",
        asset.load("mev_yolodeep:assets/dstest2_sgie1_config.txt").filename,
    )
    # ------ Secondary Classifier 1 ------#

    # ------ Secondary Classifier 2 ------#
    sgie2.set_property(
        "config-file-path",
        asset.load("mev_yolodeep:assets/dstest2_sgie2_config.txt").filename,
    )
    # ------ Secondary Classifier 2 ------#

    # ------ Secondary Classifier 3 ------#
    sgie3.set_property(
        "config-file-path",
        asset.load("mev_yolodeep:assets/dstest2_sgie3_config.txt").filename,
    )
    # ------ Secondary Classifier 3 ------#

    # ------ Start of tracker test code ------ #
    # Set properties of tracker
    trackfig = configparser.ConfigParser()
    trackfig.read(asset.load("mev_yolodeep:assets/dstest2_tracker_config.txt").filename)
    trackfig.sections()

    for key in trackfig["tracker"]:
        if key == "tracker-width":
            tracker_width = trackfig.getint("tracker", key)
            tracker.set_property("tracker-width", tracker_width)
        if key == "tracker-height":
            tracker_height = trackfig.getint("tracker", key)
            tracker.set_property("tracker-height", tracker_height)
        if key == "gpu-id":
            tracker_gpu_id = trackfig.getint("tracker", key)
            tracker.set_property("gpu_id", tracker_gpu_id)
        if key == "ll-lib-file":
            tracker_ll_lib_file = trackfig.get("tracker", key)
            tracker.set_property("ll-lib-file", tracker_ll_lib_file)
        if key == "ll-config-file":
            tracker_ll_config_file = trackfig.get("tracker", key)
            tracker.set_property("ll-config-file", tracker_ll_config_file)
    # ------ End of tracker test code ------ #

    print("Adding elements to Pipeline \n")
    pipeline.add(source)
    pipeline.add(h264parser)
    pipeline.add(decoder)
    pipeline.add(streammux)
    pipeline.add(pgie)
    # ------ Start of tracker test code ------ #
    pipeline.add(tracker)
    # ------ End of tracker test code ------ #
    # ------ Secondary Classifiers ------#
    pipeline.add(sgie1)
    pipeline.add(sgie2)
    pipeline.add(sgie3)
    # ------ Secondary Classifiers ------#
    pipeline.add(nvvidconv)
    pipeline.add(nvosd)
    pipeline.add(sink)

    # we link the elements together
    # file-source -> h264-parser -> nvh264-decoder -> nvinfer -> nvvidconv -> nvosd -> video-renderer
    source.link(h264parser)
    h264parser.link(decoder)

    sinkpad = streammux.get_request_pad("sink_0")
    if not sinkpad:
        sys.stderr.write(" Unable to get the sink pad of streammux \n")
    srcpad = decoder.get_static_pad("src")
    if not srcpad:
        sys.stderr.write(" Unable to get source pad of decoder \n")
    srcpad.link(sinkpad)
    streammux.link(pgie)
    # ------ Start of test code ------ #
    pgie.link(tracker)
    tracker.link(sgie1)
    sgie1.link(sgie2)
    sgie2.link(sgie3)
    # ------ End of test code ------ #
    sgie3.link(nvvidconv)
    nvvidconv.link(nvosd)
    nvosd.link(sink)

    # create an event loop and feed gstreamer bus mesages to it
    loop = GLib.MainLoop()
    bus = pipeline.get_bus()
    bus.add_signal_watch()
    bus.connect("message", bus_call, loop)

    # Lets add probe to get informed of the meta data generated, we add probe to
    # the sink pad of the osd element, since by that time, the buffer would have
    # had got all the metadata.
    osdsinkpad = nvosd.get_static_pad("sink")
    if not osdsinkpad:
        sys.stderr.write(" Unable to get sink pad of nvosd \n")

    osdsinkpad.add_probe(Gst.PadProbeType.BUFFER, osd_sink_pad_buffer_probe, 0)
    GLib.timeout_add(1000, perf_data.perf_print_callback)
    # start play back and listen to events
    pipeline.set_state(Gst.State.PLAYING)
    try:
        loop.run()
    except:
        pass
    # cleanup
    pipeline.set_state(Gst.State.NULL)


def run():
    config = parse_args()
    sys.exit(main(config))


if __name__ == "__main__":
    run()
